#include <iostream>  // std::cout
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <stl/formatlist.hpp>

// This is needed for the std::map examples
// NOTE: Is there a better way to do this? io_format?

namespace std
{
   template< typename CharT, class TraitsT, typename T1, typename T2 >
   inline basic_ostream< CharT, TraitsT > & operator<<
   (
      basic_ostream< CharT, TraitsT > & os,
      pair< T1, T2 > & p
   )
   {
      return( os << "( " << p.first << ", " << p.second << " )" );
   }
}

/*
Output From The Program:
=======================

basic output:

vector< int > = [ 41, 18467, 6334, 26500, 19169, 15724, 11478, 29358, 26962,
24464 ]
list< char > = [ A, B, C ]
map< string, string > = [ ( Abscheid Nehmen, Farewell ), ( Engel, Angel ),
( Mas Alla, Beyond ), ( Wenn Das Liebe Ist, If This Is Love ) ]
int[ 4 ] = [ 3, 6, 9, 12 ]
char * = [ J, e, a, n, e, t, t, e,  , B, i, e, d, e, r, m, a, n, n ]
std::string = [ W, a, r, u, m,  , b, i, s, t,  , d, u,  , n, i, c, h, t,  ,
 d, a, ? ]
int[ 3 ][ 3 ] = [ 0012FE48, 0012FE54, 0012FE60 ]

formatting options:

vector< int > = ( 41, 18467, 6334, 26500, 19169, 15724, 11478, 29358, 26962,
24464 )
vector< int > = [ 41 | 18467 | 6334 | 26500 | 19169 | 15724 | 11478 | 29358 |
26962 | 24464 ]
map< string, string > =
( Abscheid Nehmen, Farewell )
( Engel, Angel )
( Mas Alla, Beyond )
( Wenn Das Liebe Ist, If This Is Love )
int[ 4 ] = [ 3 ; 6 ; 9 ; 12 ]
char * = [J; e; a; n; e; t; t; e;  ; B; i; e; d; e; r; m; a; n; n]
std::string = W|a|r|u|m| |b|i|s|t| |d|u| |n|i|c|h|t| |d|a|?

sub-list:

std::string = "b-i-s-t- -d-u- -n-i-c-h-t- -d-a-?"
*/

int __cdecl main()
{
   // std::vector

   std::vector< int >                  v( 10 );
   std::generate( v.begin(), v.end(), std::rand );

   // std::list

   std::list< char >                   l;
   l.push_back( 'A' );
   l.push_back( 'B' );
   l.push_back( 'C' );

   // std::map

   std::map< std::string, std::string > m;
   m[ "Abscheid Nehmen"    ] = "Farewell";
   m[ "Engel"              ] = "Angel";
   m[ "Mas Alla"           ] = "Beyond";
   m[ "Wenn Das Liebe Ist" ] = "If This Is Love";

   // std::string and arrays

   const char *                        str = "Jeanette Biedermann";
   std::string                         s   = "Warum bist du nicht da?";

   int                                 i[ 4 ] = { 3, 6, 9, 12 };
   int                                 a[ 3 ][ 3 ] =
   {
      { 1, 0, 0 },
      { 0, 1, 0 },
      { 0, 0, 1 }
   };

   // basic output examples
   // NOTE: Because 2D arrays are not supported, the 2D example does not work as intended.

   std::cout << "basic output:" << '\n' << '\n';

   std::cout << "vector< int > = "         << stl::formatlist( v ) << '\n';
   std::cout << "list< char > = "          << stl::formatlist( l ) << '\n';
   std::cout << "map< string, string > = " << stl::formatlist( m ) << '\n';
   std::cout << "int[ 4 ] = "              << stl::formatlist( i, i + 4 ) << '\n';
   std::cout << "char * = "                << stl::formatlist( str, str + std::strlen( str )) << '\n';
   std::cout << "std::string = "           << stl::formatlist( s ) << '\n';
   std::cout << "int[ 3 ][ 3 ] = "         << stl::formatlist( a, a + 3 ) << '\n';

   // list format control
   
   std::cout << '\n' << "formatting options:" << '\n' << '\n';

   std::cout << "vector< int > = " << stl::formatlist( v ).format( '(', ')' ) << '\n';
   std::cout << "vector< int > = " << stl::formatlist( v ).format( '|' ).space( true, true ) << '\n';

   std::cout << "map< string, string > = "
             << stl::formatlist( m ).format( '\n', '\0', '\n' ).space( false, false, false ) << '\n';

   std::cout << "int[ 4 ] = "
             << stl::formatlist( i, i + 4 ).format( ';' ).space( true, true )
             << '\n';

   std::cout << "char * = "
             << stl::formatlist( str, str + std::strlen( str )).format( ';' ).space( false )
             << '\n';

   std::cout << "std::string = "
             << stl::formatlist( s ).format( '\0', '\0', '|' ).space( false, false, false )
             << '\n';
   
   std::cout << '\n' << "sub-list:" << '\n' << '\n';

   std::cout << "std::string = "
             << stl::formatlist( s.begin() + 6, s.end()).space( false, false, false ).format( '"', '"', '-' )
             << '\n';

   return( 0 );
}
